#ifndef SRPC_BITS_H
#define SRPC_BITS_H

#include <algorithm>
#include <climits>
#include <cassert>

namespace srpc {

/// get bit-count of type T
template <typename T>
struct Bits
{
    static const int size = (sizeof(T) * CHAR_BIT);
};


/// get bit-count of type bool
template <>
struct Bits<bool>
{
    static const int size = 1;
};


/// Ʈ ä  Ʈ Ѵ
template <typename T>
inline T toBytes(T bitCount)
{
    if (bitCount < 0) {
        return 0;
    }
    return (bitCount + (CHAR_BIT - 1)) / CHAR_BIT;    
}


/// Ʈ  ִ Ʈΰ?
template <typename T>
inline bool isEnoughBits(int bitCount)
{
    return (0 < bitCount) && (bitCount <= Bits<T>::size);
}


/// copy [src, src + size) to [dest, ...)
template<typename InIt, typename OutIt>
inline void copyVector(InIt src, OutIt dest, size_t size)
{
#if _MSC_VER >= 1400 // MSVC++ 8.0
    stdext::unchecked_copy(src, src + size, dest);
#else
    std::copy(src, src + size, dest);
#endif
}

} // namespace srpc

#endif // SRPC_BITS_H
