#include "stdafx.h"
#include <srpc/RpcReceiver.h>
#include <srpc/RpcNetwork.h>
#include <srpc/IStream.h>

namespace srpc {

RpcReceiver::RpcReceiver(RpcNetwork* rpcNetwork) :
    rpcNetwork_(0)
{
    if (rpcNetwork != 0) {
        setRpcNetwork(rpcNetwork);
    }
}


bool RpcReceiver::handle(const RpcId& rpcId, IStream& istream,
    const void* rpcHint)
{
    RpcEvent* rpcEvent = lookupEvent(rpcId);
    if (! rpcEvent)
        return false;

    return handleEvent(*rpcEvent, istream, rpcHint);
}


void RpcReceiver::setRpcNetwork(RpcNetwork* rpcNetwork)
{
    assert(rpcNetwork != 0);

    rpcNetwork_ = rpcNetwork;
    if (rpcNetwork_ != 0) {
        rpcNetwork_->registerRpcReceiver(this);
    }
}


void RpcReceiver::resetRpcNetwork()
{
    if (rpcNetwork_ != 0) {
        rpcNetwork_->unregisterRpcReceiver(this);
        rpcNetwork_ = 0;
    }
}


RpcEvent* RpcReceiver::lookupEvent(const RpcId& rpcId) const
{
    const RpcEventMap& eventMap = getEventMap();

    return eventMap.getRpcEvent(rpcId);
}


bool RpcReceiver::handleEvent(RpcEvent& rpcEvent, IStream& istream,
    const void* rpcHint)
{
    // CAUTION: ߻  ߸ ̺ ϴ  ϱ
    //  ȸ(indirection)
    dispatch(rpcEvent, istream, rpcHint);

    istream.align();

    return true;
}

} // namespace srpc
