#ifndef SRPC_ISTREAM_H
#define SRPC_ISTREAM_H

#include "Types.h"
#include "StringTypes.h"
#include "utility/Bits.h"
#ifdef _MSC_VER
#  pragma warning (push)
#  pragma warning (disable: 4819)
#endif
#include <boost/type_traits/integral_constant.hpp>
#include <boost/utility.hpp>
#ifdef _MSC_VER
#  pragma warning (pop)
#endif
#include <string>
#include <cassert>

namespace srpc {

/** @addtogroup serialization
* @{
*/

/**
 * @class IStream
 *
 * Input stream.
 * - CAUTION:  Ÿ  Ʈ о´
 */
class IStream : public boost::noncopyable
{
public:
    virtual ~IStream() {}

    /// Ʈ۷  64Ʈ ȣ   Ÿ о ´
    virtual void read(UInt64& value, int bitCount = Bits<UInt64>::size) = 0;

    /// Ʈ۷  32Ʈ ȣ   Ÿ о ´
    virtual void read(UInt32& value, int bitCount = Bits<UInt32>::size) = 0;

    /// Ʈ۷  16Ʈ ȣ   Ÿ о ´
    virtual void read(UInt16& value, int bitCount = Bits<UInt16>::size) = 0;

    /// Ʈ۷  8Ʈ ȣ   Ÿ о ´
    virtual void read(UInt8& value, int bitCount = Bits<UInt8>::size) = 0;

    /// Ʈ۷  64Ʈ ȣ ִ  Ÿ о ´
    virtual void read(Int64& value, int bitCount = Bits<Int64>::size) = 0;

    /// Ʈ۷  32Ʈ ȣ ִ  Ÿ о ´
    virtual void read(Int32& value, int bitCount = Bits<Int32>::size) = 0;

    /// Ʈ۷  16Ʈ ȣ ִ  Ÿ о ´
    virtual void read(Int16& value, int bitCount = Bits<Int16>::size) = 0;

    /// Ʈ۷  8Ʈ ȣ ִ  Ÿ о ´
    virtual void read(Int8& value, int bitCount = Bits<Int8>::size) = 0;

    /// Ʈ۷  ο︰ о ´
    virtual void read(bool& value, int bitCount = Bits<bool>::size) = 0;

    /// Ʈ۷  32Ʈ ε Ҽ Ÿ о ´
    virtual void read(Float32& value, int bitCount = Bits<Int32>::size) = 0;

    /// ִ maxLength  ڿ о´
    virtual void read(String& value, size_t maxLength,
        int sizeBitCount) = 0;

    /// ִ maxLength  ڿ о´
    virtual void read(WString& value, size_t maxLength,
        int sizeBitCount) = 0;

    /**
     * Ʈ۷  raw buffer о ´
     * @param buffer Է . ݵ length ̻ ޸𸮸 Ȯؾ Ѵ.
     * @param length о Ʈ 
     */
    virtual void read(void* buffer, UInt16 length) = 0;

    /// Ʈ Ʈ  Ѵ(ۿ ִ Ʈ )
    virtual void align() = 0;

    /**
     * Ʈ ʱȭѴ.
     * @param resetBuffer ۸ ʱȭ ΰ?
     */
    virtual void reset(bool resetBuffer) = 0;

    /// ۿ  ִ   ũ(Ʈ ) ´
    virtual int getTotalBitCount() const = 0;

    /// ۿ  ִ   ũ(Ʈ ) ´
    virtual int getTotalSize() const = 0;

    template <typename T>
    void do_read(T& value, int sizeBitCount, const boost::false_type&) {
        read(value, sizeBitCount);
    }

    template <typename T>
    void do_read(T& value, int sizeBitCount, const boost::true_type&) {
        Int32 temp;
        read(temp, sizeBitCount);
        value = static_cast<T>(temp);
    }
};

/** @} */ // addtogroup serialization

} // namespace srpc

#endif // SRPC_ISTREAM_H
