#include "stdafx.h"
#include <nsrpc/RpcClientSession.h>
#include <nsrpc/detail/SessionRpcNetwork.h>
#include <nsrpc/detail/PacketCoder.h>
#include <nsrpc/PacketSeedExchanger.h>

namespace nsrpc
{

// = RpcClientSession

IMPLEMENT_SRPC_EVENT_DISPATCHER(RpcClientSession);

#ifdef _MSC_VER
#  pragma warning (push)
#  pragma warning (disable: 4355)
#endif

RpcClientSession::RpcClientSession(ACE_Reactor* reactor,
    PacketCoderFactory* packetCoderFactory, bool useBitPacking) :
    ClientSession(reactor, packetCoderFactory),
    rpcNetwork_(new SessionRpcNetwork(useBitPacking)),
    seedExchanger_(PacketSeedExchanger::createForClient())
{
    assert(rpcNetwork_.get() != 0);

    rpcNetwork_->initialize(this, this);

    srpc::RpcReceiver::setRpcNetwork(rpcNetwork_.get());
    srpc::RpcForwarder::setRpcNetwork(rpcNetwork_.get());

    seedExchanger_->initialize(&getPacketCoder(), rpcNetwork_.get(),
        &getLock());
}

#ifdef _MSC_VER
#  pragma warning (pop)
#endif

RpcClientSession::~RpcClientSession()
{
}


SessionRpcNetwork* RpcClientSession::getRpcNetwork()
{
    return rpcNetwork_.get();
}


void RpcClientSession::sendingFailed()
{
    onSendError();
    disconnect(true);
}


void RpcClientSession::receivingFailed()
{
    onReceiveError();
    disconnect(true);
}


void RpcClientSession::onConnected()
{
    rpcNetwork_->initialize(this, this);
}


void RpcClientSession::onDisconnected()
{
    rpcNetwork_->disable();
}


void RpcClientSession::onMessageArrived(CsMessageType messageType)
{
    if (! rpcNetwork_->messageArrived(messageType)) {
        receivingFailed();
    }
}

// = SessionRpcNetworkCallback overriding

void RpcClientSession::sendNow(ACE_Message_Block& mblock,
    CsMessageType messageType)
{
    sendMessage(mblock, messageType);
}


void RpcClientSession::marshalingErrorOccurred()
{
    sendingFailed();
}


void RpcClientSession::unmarshalingErrorOccurred()
{
    receivingFailed();
}

} // namespace nsrpc
