#ifndef NSRPC_PACKETCODER_H
#define NSRPC_PACKETCODER_H

#include "../config/config.h"
#include "Protocol.h"
#include <srpc/utility/Endian.h>
#include <srpc/StringTypes.h>
#include <ace/Message_Block.h>
#include <boost/noncopyable.hpp>

namespace nsrpc
{

/** @addtogroup protocol
* @{
*/

/**
 * @class PacketCoder
 * Ŷ  ϰų оδ.
 * - Ŷ (, ȣȭ)Ϸ  Ŭ ӹ޾ óؾ Ѵ.
 */
class NSRPC_API PacketCoder : public boost::noncopyable
{
public:
    typedef srpc::String Seed;
public:
    virtual ~PacketCoder() {}

    /// ʱȭ Ѵ.
    virtual void reset() = 0;

    /// ȣȭ  õ尪 ´.
    virtual void generateCipherSeed(Seed& encryptSeed,
        Seed& decryptSeed) const = 0;

    /// base  ȣȭ Ű Ѵ.
    virtual Seed generateCipherSeed(srpc::UInt32 base) const;

    /// ȣȭ  õ尪 Ѵ
    virtual void setEncryptSeed(const Seed& seed) = 0;

    /// ȣȭ  õ尪 Ѵ
    virtual void setDecryptSeed(const Seed& seed) = 0;

    /**
     * mblock   о´.
     * @pre isHeadOfPacket()
     * @post isHeadOfPacket() && header.isValid()
     */
    virtual bool readHeader(AbstractPacketHeader& header,
        const ACE_Message_Block& mblock) const;

    /*
     * Ŷ ڵѴ.
     * @param header header.bodySize_ 0̸ ο  ä.
     * @pre isHeadOfPacket()
     * @post isHeadOfPacket() && header.isValid()
     */
    virtual bool encode(ACE_Message_Block& mblock,
        AbstractPacketHeader& header);

    /**
     * Ŷ ڵѴ.
     * @pre isHeadOfPacket()
     * @post isHeadOfPacket()
     */
    virtual bool decode(ACE_Message_Block& mblock);

    /// ȣŰ ȿ Ⱓ Ѵ
    virtual void extendCipherKeyTimeLimit() = 0;

    /// Ŷ  ũ
    virtual size_t getHeaderSize() const = 0;

    ///< Ŷ ⺻ ޸ ũ
    virtual size_t getDefaultPacketSize() const = 0;

    ///< Ŷ Ǯ ũ
    virtual size_t getDefaultPacketPoolSize() const = 0;

    /// ȣȭ ʱ Ű ȯؾ ϴ°?
    virtual bool shouldExchangeCipherSeed() const = 0;

    /// ȣȭ Ű ȿⰣ Ǿ°?
    virtual bool isCipherKeyExpired() const = 0;
public:
    /// Ŷ    Ѵ.
    void reserveHeader(ACE_Message_Block& mblock);

    /// rd_ptr() packet body ġŲ.
    void advanceToBody(ACE_Message_Block& mblock) {
        mblock.rd_ptr(mblock.base() + getHeaderSize());
    }

    bool isValidPacket(const ACE_Message_Block& mblock) const {
        return mblock.length() > getHeaderSize();
    }

    bool isValidHeader(const AbstractPacketHeader& header,
        ACE_Message_Block& mblock) const {
        return (mblock.length() - getHeaderSize()) ==
            static_cast<size_t>(header.bodySize_);
    }
protected:
    BodySize getBodySize(const ACE_Message_Block& mblock) const {
        return static_cast<BodySize>(mblock.length() - getHeaderSize());
    }
private:
    bool isValidPacketHeader(const ACE_Message_Block& mblock) const {
        return mblock.length() >= getHeaderSize();
    }

    bool isHeadOfPacket(const ACE_Message_Block& mblock) const {
        return mblock.rd_ptr() == mblock.base();
    }
};

/** @} */ // addtogroup protocol

} // namespace nsrpc

#endif // !defined(NSRPC_PACKETCODER_H)
